# Generated by Django 4.2.15 on 2024-10-04 16:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('persona_integration', '0006_alter_verificationattempt_user'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verificationattempt',
            name='event_created_at',
            field=models.DateTimeField(null=True),
        ),
        migrations.AlterField(
            model_name='verificationattempt',
            name='inquiry_id',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='verificationattempt',
            name='status',
            field=models.CharField(choices=[('initialized', 'initialized'), ('created', 'created'), ('pending', 'pending'), ('expired', 'expired'), ('approved', 'approved'), ('declined', 'declined')], max_length=64),
        ),
    ]
