# Generated by Django 4.2.14 on 2024-08-28 20:27

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('persona_integration', '0005_verificationattempt_platform_verification_attempt_id'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verificationattempt',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='persona_verificationattempt_set', to=settings.AUTH_USER_MODEL),
        ),
    ]
