# Generated by Django 4.2.14 on 2024-08-07 18:43

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('persona_integration', '0003_verificationattempt_event_created_at_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='userpersonaaccount',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, unique=True),
        ),
    ]
