# Generated by Django 4.2.14 on 2024-08-28 17:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('persona_integration', '0004_alter_userpersonaaccount_user'),
    ]

    operations = [
        migrations.AddField(
            model_name='verificationattempt',
            name='platform_verification_attempt_id',
            field=models.PositiveIntegerField(null=True),
        ),
    ]
