# Generated by Django 4.2.14 on 2024-07-31 10:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('persona_integration', '0002_verificationattempt'),
    ]

    operations = [
        migrations.AddField(
            model_name='verificationattempt',
            name='event_created_at',
            field=models.DateTimeField(default=None),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='verificationattempt',
            name='inquiry_id',
            field=models.CharField(default=None, max_length=255, unique=True),
            preserve_default=False,
        ),
    ]
